@@ -1,6 +1,6 @@
 package App::Sets;
 {
-  $App::Sets::VERSION = '0.973_02';
+  $App::Sets::VERSION = '0.973_03';
 }
 
 # ABSTRACT: set operations in Perl
@@ -8,7 +8,6 @@ package App::Sets;
 use strict;
 use warnings;
 use English qw( -no_match_vars );
-use IPC::Open2 qw< open2 >;
 use 5.010;
 use Getopt::Long
   qw< GetOptionsFromArray :config pass_through no_ignore_case bundling >;
@@ -16,6 +15,8 @@ use Pod::Usage qw< pod2usage >;
 use Log::Log4perl::Tiny qw< :easy :dead_if_first LOGLEVEL >;
 use App::Sets::Parser;
 use App::Sets::Iterator;
+use App::Sets::Operations;
+use App::Sets::Sort qw< sort_filehandle >;
 
 my %config = (
    loglevel => 'INFO',
@@ -123,20 +124,14 @@ sub expression {
    my ($expression) = @_;
    if (ref $expression) {    # operation
       my ($op, $l, $r) = @$expression;
-      return __PACKAGE__->can($op)->(expression($l), expression($r));
+      my $sub = App::Sets::Operations->can($op);
+      return $sub->(expression($l), expression($r));
    }
    else {                    # plain file
       return file($expression);
    }
 } ## end sub expression
 
-sub _sort_filehandle {
-   my ($filename) = @_;
-   open my $fh, '-|', 'sort', '-u', $filename
-     or LOGDIE "open() sort -u '$filename': $OS_ERROR";
-   return $fh;
-} ## end sub _sort_filehandle
-
 sub file {
    my ($filename) = @_;
    LOGDIE "invalid file '$filename'\n"
@@ -147,7 +142,7 @@ sub file {
       if (!-e $cache_filename) {    # generate cache file
          WARN "generating cached sorted file "
            . "'$cache_filename', might wait a bit...";
-         my $ifh = _sort_filehandle($filename);
+         my $ifh = sort_filehandle($filename);
          open my $ofh, '>', $cache_filename
            or LOGDIE "open('$cache_filename') for output: $OS_ERROR";
          while (<$ifh>) {
@@ -169,7 +164,7 @@ sub file {
    } ## end if ($config{sorted})
    else {
       INFO "opening '$filename' and sorting on the fly";
-      $fh = _sort_filehandle($filename);
+      $fh = sort_filehandle($filename);
    }
    return App::Sets::Iterator->new(
       sub {
@@ -182,81 +177,6 @@ sub file {
    );
 } ## end sub file
 
-sub intersect {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while ('necessary') {
-            $lh //= $l->drop() // last;
-            $rh //= $r->drop() // last;
-            if ($lh eq $rh) {
-               my $retval = $lh;
-               $lh = $rh = undef;
-               return $retval;
-            }
-            elsif ($lh gt $rh) {
-               $rh = undef;
-            }
-            else {
-               $lh = undef;
-            }
-         } ## end while ('necessary')
-         return undef;
-      }
-   );
-} ## end sub intersect
-
-sub union {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
-            if ($lh eq $rh) {
-               $r->drop();
-               return $l->drop();
-            }
-            elsif ($lh lt $rh) {
-               return $l->drop();
-            }
-            else {
-               return $r->drop();
-            }
-         } ## end while (defined($lh = $l->head...
-         while (defined($lh = $l->drop())) {
-            return $lh;
-         }
-         while (defined($rh = $r->drop())) {
-            return $rh;
-         }
-         return undef;
-      }
-   );
-} ## end sub union
-
-sub minus {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
-            if ($lh eq $rh) {    # shared, drop both
-               $r->drop();
-               $l->drop();
-            }
-            elsif ($lh lt $rh) {    # only in left, OK!
-               return $l->drop();
-            }
-            else {                  # only in right, go on
-               $r->drop();
-            }
-         } ## end while (defined($lh = $l->head...
-         return $l->drop();
-      }
-   );
-} ## end sub minus
-
 1;
 
 __END__
@@ -269,7 +189,7 @@ App::Sets - set operations in Perl
 
 =head1 VERSION
 
-version 0.973_02
+version 0.973_03
 
 =head1 AUTHOR
 
